/* Generated from Java with JSweet 2.2.0-SNAPSHOT - http://www.jsweet.org */
namespace org.ta4j.core {
    /**
     * Sequence of {@link Bar} separated by a predefined period (e.g. 15 minutes, 1 day, etc.)
     * </p>
     * Notably, a {@link TimeSeries} can be:
     * <ul>
     * <li>the base of {@link Indicator} calculations
     * <li>constrained between begin and end indexes (e.g. for some backtesting cases)
     * <li>limited to a fixed number of bars (e.g. for actual trading)
     * </ul>
     * @class
     */
    export interface TimeSeries {
        /**
         * @return {string} the name of the series
         */
        getName() : string;

        /**
         * @param {number} i an index
         * @return {*} the bar at the i-th position
         */
        getBar(i : number) : org.ta4j.core.Bar;

        /**
         * @return {*} the first bar of the series
         */
        getFirstBar() : org.ta4j.core.Bar;

        /**
         * @return {*} the last bar of the series
         */
        getLastBar() : org.ta4j.core.Bar;

        /**
         * @return {number} the number of bars in the series
         */
        getBarCount() : number;

        /**
         * @return {boolean} true if the series is empty, false otherwise
         */
        isEmpty() : boolean;

        /**
         * Warning: should be used carefully!
         * <p>
         * Returns the raw bar data.
         * It means that it returns the current List object used internally to store the {@link Bar}.
         * It may be:
         * - a shortened bar list if a maximum bar count has been set
         * - a extended bar list if it is a constrained time series
         * @return {*[]} the raw bar data
         */
        getBarData() : Array<org.ta4j.core.Bar>;

        /**
         * @return {number} the begin index of the series
         */
        getBeginIndex() : number;

        /**
         * @return {number} the end index of the series
         */
        getEndIndex() : number;

        /**
         * @return {string} the description of the series period (e.g. "from 12:00 21/01/2014 to 12:15 21/01/2014")
         */
        getSeriesPeriodDescription() : string;

        /**
         * Sets the maximum number of bars that will be retained in the series.
         * <p>
         * If a new bar is added to the series such that the number of bars will exceed the maximum bar count,
         * then the FIRST bar in the series is automatically removed, ensuring that the maximum bar count is not exceeded.
         * @param {number} maximumBarCount the maximum bar count
         */
        setMaximumBarCount(maximumBarCount : number);

        /**
         * @return {number} the maximum number of bars
         */
        getMaximumBarCount() : number;

        /**
         * @return {number} the number of removed bars
         */
        getRemovedBarsCount() : number;

        /**
         * Adds a bar at the end of the series.
         * <p>
         * Begin index set to 0 if if wasn't initialized.<br>
         * End index set to 0 if if wasn't initialized, or incremented if it matches the end of the series.<br>
         * Exceeding bars are removed.
         * @param {*} bar the bar to be added
         * @see TimeSeries#setMaximumBarCount(int)
         */
        addBar(bar : org.ta4j.core.Bar);

        /**
         * Returns a new TimeSeries implementation that is a subset of this TimeSeries implementation.
         * It holds a copy of all {@link Bar} between <tt>startIndex</tt> (inclusive) and <tt>endIndex</tt> (exclusive)
         * of this TimeSeries.
         * The indices of this TimeSeries and the new subset TimeSeries can be different. I. e. index 0 of the new TimeSeries will
         * be index <tt>startIndex</tt> of this TimeSeries.
         * If <tt>startIndex</tt> < this.seriesBeginIndex the new TimeSeries will start with the first available Bar of this TimeSeries.
         * If <tt>endIndex</tt> > this.seriesEndIndex the new TimeSeries will end at the last available Bar of this TimeSeries
         * @param {number} startIndex the startIndex
         * @param {number} endIndex the endIndex
         * @return {*} a new BaseTimeSeries with Bars from startIndex to endIndex-1
         * @throws IllegalArgumentException e.g. if endIndex < startIndex
         */
        getSubSeries(startIndex : number, endIndex : number) : TimeSeries;
    }
}

