/* Generated from Java with JSweet 2.2.0-SNAPSHOT - http://www.jsweet.org */
namespace org.ta4j.core.indicators.pivotpoints {
    /**
     * Constructor.
     * <p>
     * Calculates the pivot point based on the time level parameter.
     * @param {*} series the time series with adequate endTime of each bar for the given time level.
     * @param {org.ta4j.core.indicators.pivotpoints.TimeLevel} timeLevel the corresponding {@link TimeLevel} for pivot calculation:
     * <ul>
     * <li>1-, 5-, 10- and 15-minute charts use the prior days high, low and close: <b>timeLevelId</b> = TimeLevel.DAY</li>
     * <li>30- 60- and 120-minute charts use the prior week's high, low, and close: <b>timeLevelId</b> =  TimeLevel.WEEK</li>
     * <li>Pivot Points for daily charts use the prior month's high, low and close: <b>timeLevelId</b> = TimeLevel.MONTH</li>
     * <li>Pivot Points for weekly and monthly charts use the prior year's high, low and close: <b>timeLevelId</b> = TimeLevel.YEAR (= 4)</li>
     * <li> If you want to use just the last bar data: <b>timeLevelId</b> = TimeLevel.BARBASED</li>
     * </ul>
     * The user has to make sure that there are enough previous bars to calculate correct pivots at the first bar that matters. For example for PIVOT_TIME_LEVEL_ID_MONTH
     * there will be only correct pivot point values (and reversals) after the first complete month
     * @class
     * @extends org.ta4j.core.indicators.RecursiveCachedIndicator
     */
    export class PivotPointIndicator extends org.ta4j.core.indicators.RecursiveCachedIndicator<org.ta4j.core.Decimal> {
        /*private*/ timeLevel : org.ta4j.core.indicators.pivotpoints.TimeLevel;

        public constructor(series : org.ta4j.core.TimeSeries, timeLevel : org.ta4j.core.indicators.pivotpoints.TimeLevel) {
            super(series);
            if(this.timeLevel===undefined) this.timeLevel = null;
            this.timeLevel = timeLevel;
        }

        /**
         * 
         * @param {number} index
         * @return {org.ta4j.core.Decimal}
         */
        calculate(index : number) : org.ta4j.core.Decimal {
            return this.calcPivotPoint(this.getBarsOfPreviousPeriod(index));
        }

        /*private*/ calcPivotPoint(barsOfPreviousPeriod : Array<number>) : org.ta4j.core.Decimal {
            if(/* isEmpty */(barsOfPreviousPeriod.length == 0)) return org.ta4j.core.Decimal.NaN_$LI$();
            let bar : org.ta4j.core.Bar = this.getTimeSeries().getBar(/* get */barsOfPreviousPeriod[0]);
            let close : org.ta4j.core.Decimal = bar.getClosePrice();
            let high : org.ta4j.core.Decimal = bar.getMaxPrice();
            let low : org.ta4j.core.Decimal = bar.getMinPrice();
            for(let index143=0; index143 < barsOfPreviousPeriod.length; index143++) {
                let i = barsOfPreviousPeriod[index143];
                {
                    high = (this.getTimeSeries().getBar(i).getMaxPrice()).max(high);
                    low = (this.getTimeSeries().getBar(i).getMinPrice()).min(low);
                }
            }
            return (high.plus(low).plus(close)).dividedBy(org.ta4j.core.Decimal.THREE_$LI$());
        }

        /**
         * Calculates the indices of the bars of the previous period
         * @param {number} index index of the current bar
         * @return {number[]} list of indices of the bars of the previous period
         */
        public getBarsOfPreviousPeriod(index : number) : Array<number> {
            let previousBars : Array<number> = <any>([]);
            if(this.timeLevel === org.ta4j.core.indicators.pivotpoints.TimeLevel.BARBASED) {
                /* add */(previousBars.push(Math.max(0, index - 1))>0);
                return previousBars;
            }
            if(index === 0) {
                return previousBars;
            }
            let currentBar : org.ta4j.core.Bar = this.getTimeSeries().getBar(index);
            while((index - 1 >= this.getTimeSeries().getBeginIndex() && this.getPeriod(this.getTimeSeries().getBar(index - 1)) === this.getPeriod(currentBar))) {
                index--;
            };
            let previousPeriod : number = this.getPreviousPeriod(currentBar, index - 1);
            while((index - 1 >= this.getTimeSeries().getBeginIndex() && this.getPeriod(this.getTimeSeries().getBar(index - 1)) === previousPeriod)) {
                index--;
                /* add */(previousBars.push(index)>0);
            };
            return previousBars;
        }

        /*private*/ getPreviousPeriod(bar : org.ta4j.core.Bar, indexOfPreviousBar : number) : number {
            switch((this.timeLevel)) {
            case org.ta4j.core.indicators.pivotpoints.TimeLevel.DAY:
                let prevCalendarDay : number = bar.getEndTime().minusDays(1).getDayOfYear();
                while((this.getTimeSeries().getBar(indexOfPreviousBar).getEndTime().getDayOfYear() !== prevCalendarDay && indexOfPreviousBar > 0)) {
                    prevCalendarDay--;
                };
                return prevCalendarDay;
            case org.ta4j.core.indicators.pivotpoints.TimeLevel.WEEK:
                return bar.getEndTime().minusWeeks(1).get(java.time.temporal.IsoFields.WEEK_OF_WEEK_BASED_YEAR);
            case org.ta4j.core.indicators.pivotpoints.TimeLevel.MONTH:
                return bar.getEndTime().minusMonths(1).getMonthValue();
            default:
                return bar.getEndTime().minusYears(1).getYear();
            }
        }

        /*private*/ getPeriod(bar : org.ta4j.core.Bar) : number {
            switch((this.timeLevel)) {
            case org.ta4j.core.indicators.pivotpoints.TimeLevel.DAY:
                return bar.getEndTime().getDayOfYear();
            case org.ta4j.core.indicators.pivotpoints.TimeLevel.WEEK:
                return bar.getEndTime().get(java.time.temporal.IsoFields.WEEK_OF_WEEK_BASED_YEAR);
            case org.ta4j.core.indicators.pivotpoints.TimeLevel.MONTH:
                return bar.getEndTime().getMonthValue();
            default:
                return bar.getEndTime().getYear();
            }
        }
    }
    PivotPointIndicator["__class"] = "org.ta4j.core.indicators.pivotpoints.PivotPointIndicator";
    PivotPointIndicator["__interfaces"] = ["org.ta4j.core.Indicator","java.io.Serializable"];


}

